@@ -17,16 +17,22 @@ if (suppressWarnings(
     p <- plot(pr)
     p <- plot(pr, ci = FALSE)
     p <- plot(pr, ci = TRUE, ci.style = "dot")
-    p <- plot(pr, rawdata = TRUE)
+    p <- plot(pr, add.data = TRUE)
+    p <- plot(pr, add.data = TRUE, jitter = FALSE)
+    p <- plot(pr, colors = "bw")
+    p <- plot(pr, colors = "gs")
 
     pr <- ggpredict(fit, c("c12hour", "c172code"))
     p <- plot(pr)
     p <- plot(pr, ci = FALSE)
     p <- plot(pr, ci = TRUE, ci.style = "dot")
-    p <- plot(pr, rawdata = TRUE)
+    p <- plot(pr, add.data = TRUE)
+    p <- plot(pr, add.data = TRUE, jitter = 0)
     p <- plot(pr, facets = TRUE)
     p <- plot(pr, facets = FALSE)
     p <- plot(pr, use.theme = FALSE)
+    p <- plot(pr, colors = "bw")
+    p <- plot(pr, colors = "gs")
   })
 
 }
